searchNodes=[{"doc":"This is the main Eventful module. Eventful makes it easy to define your state machines on a given field on your schema. You can even have multiple state machines on a single schema. Let's define the Event module. defmodule MyApp.Post.Event do alias MyApp . { Post , User } use Eventful , parent : { :post , Post } , actor : { :user , User } handle ( :transitions , using : Post.Transitions ) end You will need to define a Transitions module for MyApp.Post defmodule MyApp.Post.Transitions do use Eventful.Transition , repo : MyApp.Repo @behaviour Eventful.Handler alias MyApp.Post Post |&gt; transition ( [ from : &quot;created&quot; , to : &quot;published&quot; , via : &quot;publish&quot; , fn changes -&gt; transit ( changes ) end ) Post |&gt; transition ( [ from : &quot;created&quot; , to : &quot;deleted&quot; , via : &quot;delete&quot; , fn changes -&gt; transit ( changes ) end ) end In this example we're defining Transitions on :current_state . Events will be stored using the Event schema. defmodule MyApp.Post do use Ecto.Schema import Ecto.Changeset use Eventful.Transitable alias __MODULE__ . Event alias __MODULE__ . Transitions Transitions |&gt; governs ( :current_state , on : Event ) schema &quot;posts&quot; do field :current_state , :string , default : &quot;created&quot; end end","ref":"Eventful.html","title":"Eventful","type":"module"},{"doc":"You can use the handle function to define the handler for a given domain: defmodule MyApp.Post.Event do alias MyApp . { Post , User } use Eventful , parent : { :post , Post } , actor : { :user , User } handle ( :transitions , using : Post.Transitions ) handle ( :visibilities , using : Post.Visibilities ) end","ref":"Eventful.html#handle/2","title":"Eventful.handle/2","type":"macro"},{"doc":"","ref":"Eventful.Error.html","title":"Eventful.Error","type":"module"},{"doc":"","ref":"Eventful.Error.html#t:t/0","title":"Eventful.Error.t/0","type":"type"},{"doc":"","ref":"Eventful.Exception.html","title":"Eventful.Exception","type":"module"},{"doc":"","ref":"Eventful.Handler.html","title":"Eventful.Handler","type":"behaviour"},{"doc":"","ref":"Eventful.Handler.html#c:call/3","title":"Eventful.Handler.call/3","type":"callback"},{"doc":"This modules handles processing of the metadata for the event","ref":"Eventful.Metadata.html","title":"Eventful.Metadata","type":"module"},{"doc":"","ref":"Eventful.Metadata.html#build/3","title":"Eventful.Metadata.build/3","type":"function"},{"doc":"This module is the one you'll use to define your main schema which require the state machine. For example imagine the following: defmodule MyApp.Post do use Ecto.Schema import Ecto.Changeset schema &quot;posts&quot; do field :current_state , :string , default : &quot;created&quot; end end You can add a state machine like this assuming you've setup the Event module already: defmodule MyApp.Post do use Ecto.Schema import Ecto.Changeset use Eventful.Transitable alias __MODULE__ . Event alias __MODULE__ . Transitions Transitions |&gt; governs ( :current_state , on : Event ) schema &quot;posts&quot; do field :current_state , :string , default : &quot;created&quot; end end You can also setup multiple state machines with multiple fields: defmodule MyApp.Post do use Ecto.Schema import Ecto.Changeset use Eventful.Transitable alias __MODULE__ . Event alias __MODULE__ . Transitions alias __MODULE__ . Visibilities # You can optionally use locks Transitions |&gt; governs ( :current_state , on : Event , lock : :current_state_version ) Visibilities |&gt; governs ( :visibility , on : Event ) schema &quot;posts&quot; do field :current_state , :string , default : &quot;created&quot; field :current_state_version , :integer , default : 0 field :visibility , :string , default : &quot;private&quot; end end","ref":"Eventful.Transitable.html","title":"Eventful.Transitable","type":"module"},{"doc":"The governs function allows you to define governance on each of your fields. Transitions |&gt; governs ( :current_state , on : Event )","ref":"Eventful.Transitable.html#governs/3","title":"Eventful.Transitable.governs/3","type":"macro"},{"doc":"This module providers the macros for building the transition modules Here is an example: defmodule MyApp.Post.Transitions do use Eventful.Transition , repo : MyApp.Repo @behaviour Eventful.Handler alias MyApp.Post Post |&gt; transition ( [ from : &quot;created&quot; , to : &quot;published&quot; , via : &quot;publish&quot; , fn changes -&gt; transit ( changes ) end ) Post |&gt; transition ( [ from : &quot;created&quot; , to : &quot;deleted&quot; , via : &quot;delete&quot; , fn changes -&gt; transit ( changes ) end ) end You can also define multiple state machines for example for a given post you can also have a machine for your :visibility field defmodule MyApp.Post.Visibilities do use Eventful.Transition , repo : MyApp.Repo , eventful_state : :visibility @behaviour Eventful.Handler alias MyApp.Post Post |&gt; transition ( [ from : &quot;private&quot; , to : &quot;public&quot; , via : &quot;publicize&quot; , fn changes -&gt; transit ( changes ) end ) Post |&gt; transition ( [ from : &quot;public&quot; , to : &quot;private&quot; , via : &quot;privatize&quot; , fn changes -&gt; transit ( changes ) end ) end","ref":"Eventful.Transition.html","title":"Eventful.Transition","type":"module"},{"doc":"The transition macro allows you to define the state machine for your transitions Post |&gt; transition ( [ from : &quot;created&quot; , to : &quot;published&quot; , via : &quot;publish&quot; , fn changes -&gt; transit ( changes ) end )","ref":"Eventful.Transition.html#transition/3","title":"Eventful.Transition.transition/3","type":"macro"},{"doc":"","ref":"Eventful.Transition.html#t:t/0","title":"Eventful.Transition.t/0","type":"type"},{"doc":"Handles the triggering of events You can define a trigger in the following way defmodule MyApp.Post.Triggers do use Eventful.Trigger alias MyApp.Post Post |&gt; trigger ( [ currently : &quot;published&quot; ] , fn event , post -&gt; # add your code here. end ) end","ref":"Eventful.Trigger.html","title":"Eventful.Trigger","type":"module"},{"doc":"The trigger macro function allows you to define a trigger in your trigger module Post |&gt; trigger ( [ currently : &quot;published&quot; ] , fn event , post -&gt; # add your code here. end )","ref":"Eventful.Trigger.html#trigger/3","title":"Eventful.Trigger.trigger/3","type":"macro"},{"doc":"","ref":"Eventful.Exception.UnexpectedReturnValue.html","title":"Eventful.Exception.UnexpectedReturnValue","type":"exception"},{"doc":"Eventful is a state machine library with an audit trail for your schemas. You can attach a state machine to any schema in your application. In the following we will use a blogging app as an example. Let's imagine you had a schema like the following to store your blog post. defmodule MyApp.Post do use Ecto.Schema import Ecto.Changeset schema &quot;posts&quot; do field :title , :string field :content , :string end def changeset ( resource , attrs ) do resource |&gt; cast ( attrs , [ :title , :content ] ) end end","ref":"getting-started.html","title":"Getting Started","type":"extras"},{"doc":"Let's imagine you want the ability to track the state of this post. You may have a collaboration feature where posts can be put into draft or published state, moreover you also want to track who did the transition. Let's assume you have a User schema of some kind. You could define an Event module like the following: defmodule MyApp.Post.UserEvent do alias MyApp . { Post , User } use Eventful , parent : { :post , Post } , actor : { :user , User } , table_name : &quot;post_user_events&quot; end","ref":"getting-started.html#event-schema","title":"Getting Started - Event Schema","type":"extras"},{"doc":"To make this work you'll also need to add a migration. defmodule MyApp.Repo.Migrations.CreatePostUserEvents do use Ecto.Migration def change do create table ( :post_user_events ) do add ( :name , :string , null : false ) add ( :domain , :string , null : false ) add ( :metadata , :map , default : &quot;{}&quot; ) add ( :post_id , references ( :posts , on_delete : :restrict ) , null : false ) add ( :user_id , references ( :users , on_delete : :restrict ) , null : false ) timestamps ( ) end create ( index ( :post_events , [ :post_id ] ) ) create ( index ( :post_events , [ :user_id ] ) ) end end","ref":"getting-started.html#migration","title":"Getting Started - Migration","type":"extras"},{"doc":"Next you'll need to define your Transitions this will allow you to define which states the post can transition to. defmodule MyApp.Post.Transitions do use Eventful.Transition , repo : MyApp.Repo @behaviour Eventful.Handler alias MyApp.Post Post |&gt; transition ( [ from : &quot;draft&quot; , to : &quot;published&quot; , via : &quot;publish&quot; , fn changes -&gt; transit ( changes ) end ) Post |&gt; transition ( [ from : &quot;published&quot; , to : &quot;draft&quot; , via : &quot;drafting&quot; , fn changes -&gt; transit ( changes ) end ) end Next you'll need to add some field to your Post schema which will be used to track the transitions. In this case let's add :current_state as the field and also define how the field is governed. defmodule MyApp.Post do # ... use Eventful.Transitable alias __MODULE__ . UserEvent alias __MODULE__ . Transitions Transitions |&gt; governs ( :current_state , on : UserEvent ) schema &quot;posts&quot; do field :title , :string field :content , :string field :current_state , :string , default : &quot;draft&quot; end # ... end Also be sure to define the handler in your UserEvent module defmodule MyApp.Post.UserEvent do alias MyApp . { Post , User } use Eventful , parent : { :post , Post } , actor : { :user , User } , table_name : &quot;post_user_events&quot; handle ( :transitions , using : Post.Transitions ) end You'll also need to add a migration for the post. You can use :string or if you prefer :citext for your :current_state field. defmodule MyApp.Repo.Migrations.AddCurrentStateToPosts do use Ecto.Migration def change do alter table ( :posts ) do add ( :current_state , :citext , default : &quot;draft&quot; , null : false ) end create ( index ( :posts , [ :current_state ] ) ) end end","ref":"getting-started.html#state-machine","title":"Getting Started - State Machine","type":"extras"},{"doc":"That's it! That's how your set up your first auditable state machine on your schema. You can how transition the post from state to state. { :ok , transition } = MyApp.Post.UserEvent . handle ( post , user , %{ domain : &quot;transitions&quot; , event_name : &quot;publish&quot; } )","ref":"getting-started.html#transitioning-the-state","title":"Getting Started - Transitioning the State","type":"extras"},{"doc":"Triggers are a powerful way to trigger some action that you define when a certain event occurs. Let's take a look at an example. Let's assume you have the following Transitions module. defmodule MyApp.Post.Transitions do use Eventful.Transition , repo : MyApp.Repo @behaviour Eventful.Handler alias MyApp.Post Post |&gt; transition ( [ from : &quot;draft&quot; , to : &quot;published&quot; , via : &quot;publish&quot; , fn changes -&gt; transit ( changes ) end ) Post |&gt; transition ( [ from : &quot;published&quot; , to : &quot;draft&quot; , via : &quot;drafting&quot; , fn changes -&gt; transit ( changes ) end ) end Let's say that when a post is published you would like your app to fire an email to notify someone that a post has been published . You could define a trigger like this: defmodule MyApp.Post.Triggers do alias MyApp.Post use Eventful.Trigger Post |&gt; trigger ( [ currently : &quot;published&quot; ] , fn event , post -&gt; # user code to send email end ) end You will also need to update your Transitions module like so: defmodule MyApp.Post.Transitions do use Eventful.Transition , repo : MyApp.Repo @behaviour Eventful.Handler alias MyApp.Post Post |&gt; transition ( [ from : &quot;draft&quot; , to : &quot;published&quot; , via : &quot;publish&quot; , fn changes -&gt; transit ( changes , Post.Triggers ) end ) Post |&gt; transition ( [ from : &quot;published&quot; , to : &quot;draft&quot; , via : &quot;drafting&quot; , fn changes -&gt; transit ( changes ) end ) end Now every time a post is published it will send the email notification.","ref":"triggers.html","title":"Triggers","type":"extras"},{"doc":"Guards are useful when you wish to prevent your transition from happening based on some kind of check. In your Transitions module you can define guards like so: defmodule MyApp.Post.Transitions do alias MyApp.Post @behaviour Eventful.Handler use Eventful.Transition , repo : MyApp.Repo Post |&gt; transition ( [ from : &quot;draft&quot; , to : &quot;published&quot; , via : &quot;publish&quot; ] , fn { post_changeset , _ } = changes -&gt; if MyApp . check_something ( post_changeset . data ) do transit ( changes , Post.Triggers ) else { :error , &quot;guard failed&quot; } end end ) ) end This allows you to define that a given condition computed by MyApp.check_something/1 will need to return true when publish event is invoked.","ref":"guards.html","title":"Guards","type":"extras"}]